/*
 * Copyright © 2018 Push Technology Ltd., All Rights Reserved.
 *
 * Use is subject to license terms.
 *
 * NOTICE: All information contained herein is, and remains the
 * property of Push Technology. The intellectual and technical
 * concepts contained herein are proprietary to Push Technology and
 * may be covered by U.S. and Foreign Patents, patents in process, and
 * are protected by trade secret or copyright law.
 */

#ifndef _svc_topic_add_h_
#define _svc_topic_add_h_ 1

/**
 * @file svc-topic-add.h
 *
 * Add topic feature (topic control).
 */
#include <stdlib.h>

#include "internal/service_int.h"
#include "types/topic_details_types.h"
#include "topic-specification.h"

/**
 * @brief Structure describing an "topic add" request.
 */
typedef struct svc_topic_add_request_s {
        // The topic path to be added.
        char *topic_path;
        // The topic details that describe the structure of this
        // topic specification
        TOPIC_SPECIFICATION_T *specification;
} SVC_TOPIC_ADD_REQUEST_T;

typedef enum {
        // Result code denoting the topic was
        // sucessfully added.
        TOPIC_ADD_RESULT_CREATED = 0,
        // Result code denoting the topic
        // already exists.
        TOPIC_ADD_RESULT_EXISTS = 1
} TOPIC_ADD_RESULT_CODE;

typedef struct svc_topic_add_response_s {
        TOPIC_ADD_RESULT_CODE result_code;
} SVC_TOPIC_ADD_RESPONSE_T;

typedef enum {
        /// A topic already exists with the same path but a different specification.
        TOPIC_ADD_FAIL_EXISTS_MISMATCH = 1,
        /// The supplied topic path is invalid.
        TOPIC_ADD_FAIL_INVALID_NAME = 2,
        /// Some aspect of the topic specification failed validation.
        TOPIC_ADD_FAIL_INVALID_DETAILS = 3,
        /// The client was denied permission to add a topic of the specified type at
        /// the given point in the topic hierarchy.
        TOPIC_ADD_FAIL_PERMISSIONS_FAILURE = 4,
        /// An unexpected error occurred whilst attempting to create the topic. See
        /// the server log for more details.
        TOPIC_ADD_FAIL_UNEXPECTED_ERROR = 5,
        /// Adding the topic failed because the cluster was repartitioning due to a
        /// server starting, stopping, or failing. The operation can be retried.
        TOPIC_ADD_FAIL_CLUSTER_REPARTITION = 6,
        /// Adding the topic failed because of a license limit.
        TOPIC_ADD_FAIL_EXCEEDED_LICENSE_LIMIT = 7,
        /// Adding the topic failed because a topic owned by a publisher is already
        /// bound to the parent path.
        TOPIC_ADD_FAIL_INCOMPATIBLE_PARENT = 8,
        /// Adding a slave topic failed because a topic owned by a publisher is
        /// already bound to the specified master path.
        TOPIC_ADD_FAIL_INCOMPATIBLE_MASTER = 9,
        /// Adding the topic failed because a topic is already bound to the specified
        /// path but the caller does not have the rights to manage it.
        TOPIC_ADD_FAIL_EXISTS_INCOMPATIBLE = 10
} TOPIC_ADD_FAIL_RESULT_CODE;

/**
 * @brief Callback for add_topic_from_specification().
 *
 * @param session     The current active session.
 * @param result_code The result code from the topic add request.
 * @param context     User-supplied context from the initial
 *                    add_topic_from_specification() call.
 *
 * @return HANDLER_SUCCESS or HANDLER_FAILURE.
 */
typedef int (*on_topic_add_cb)(SESSION_T *session,
                               TOPIC_ADD_RESULT_CODE result_code,
                               void *context);

/**
 * @brief Callback for add_topic_from_specification().
 *
 * @param session     The current active session.
 * @param result_code The topic add failure result code
 * @param error       The error reason. This will only be set
 *                    if this function is called through the
 *                    callback
 * @param context     User-supplied context from the initial
 *                    add_topic_from_specification() call.
 *
 * @return HANDLER_SUCCESS or HANDLER_FAILURE.
 */
typedef int (*on_topic_add_failed_cb)(SESSION_T *session,
                                      TOPIC_ADD_FAIL_RESULT_CODE result_code,
                                      const DIFFUSION_ERROR_T *error,
                                      void *context);

/**
 * add_topic_from_specification callback
 */
typedef struct add_topic_callback_s {
        /**
         * Topic added successfully (or already exists).
         */
        on_topic_add_cb on_topic_added_with_specification;
        /**
         * Topic failed to be added.
         */
        on_topic_add_failed_cb on_topic_add_failed_with_specification;
        /** 
         * Standard service error handler.
         */
        ERROR_HANDLER_T on_error;
        /**
         * Standard conversation discard callback.
         */
        DISCARD_HANDLER_T on_discard;
        /**
         * Context. Can be NULL.
         */
        void *context;
} ADD_TOPIC_CALLBACK_T;

void add_topic_from_specification(SESSION_T *session,
                                  const char *topic_path,
                                  const TOPIC_SPECIFICATION_T *specification,
                                  const ADD_TOPIC_CALLBACK_T callback);

DECLARE_SERVICE_PROTOTYPES(topic_add, SVC_TOPIC_ADD_REQUEST_T, SVC_TOPIC_ADD_RESPONSE_T)

#endif
